import mutex, thread
import sgpem

_g_mutex = mutex.mutex()

## @brief This is an adapter class which acts as a proxy
# for user-implemented policies
#
# At runtime, this class will be initialized with the
# user-implemented policy, and then it will proxy the calls
# the policy member functions on a different thread, so
# to ensure asyncronous control.
#
# Instantiated in the C++ code, it should be instantiated like:
# @code
# adapter = ScriptAdapter(UserPolicyClass)
# @endcode
class ScriptAdapter :
    _policy = None
    
    def __init__(self, policy):
        self._policy = policy()
        print 'ScriptAdapter for policy ', policy, ' loaded'

    def async_configure(self):
        print 'Calling ', self._policy, ' configure() method.'
        _g_mutex.lock(ScriptAdapter._wrap_configure, self )
        print 'Returning from configure() method'
        
    def _wrap_configure(self):
        thread.start_new_thread(ScriptAdapter._wrap_configure_callback, (self,))
    def _wrap_configure_callback(self):
        # call configure method
        self._policy.configure()
        _g_mutex.unlock()
        
    def async_sort_queue(self, event):
        _g_mutex.lock(ScriptAdapter._wrap_sort_queue, (self, event))

    def _wrap_sort_queue(self, event):
        thread.start_new_thread(ScriptAdapter._wrap_sort_queue_callback, (self, event))

    def _wrap_sort_queue_callback(self, event):
        # here we retrieve and pass the ready queue
        queue = sgpem.Scheduler.get_instance().get_ready_queue()
        self._policy.sort_queue(event, queue)
        _g_mutex.unlock()

    def async_is_preemptible(self):
        _g_mutex.lock(ScriptAdapter._wrap_is_preemptible, self)

    def _wrap_is_preemptible(self):
        thread.start_new_thread(ScriptAdapter._wrap_is_preemptible_callback, (self,))

    def _wrap_is_preemptible_callback(self):
        # FIXME : here we should return the call value
        self._policy.is_preemptible()
        _g_mutex.unlock()
    
    def async_is_timesliced(self):
        _g_mutex.lock(ScriptAdapter._wrap_is_timesliced, self)

    def _wrap_is_timesliced(self):
        thread.start_new_thread(ScriptAdapter._wrap_is_timesliced_callback, (self,))

    def _wrap_is_timesliced_callback(self):
        # FIXME : return value!
        self._policy.is_timesliced()
        _g_mutex.unlock()
